function [muhat,covhat,loglike,flag]=conditionalMode(y,mu,cov,EMiter,initialz)

% CONDITIONALMODE
%
% function [muhat,covhat]=conditionalMode(y,mu,cov)
%
% input:
%   y is a histogram of draws from the Hierarchical Logistic-Normal
%       distribution.
%   mu, cov are parameters of a Logistic-Normal.  mu is a (k-1)x1 vector
%   and cov is a (k-1)x(k-1) covariance matrix.
% output:
%   muhat,covhat is an approximate posterior mode conditioned
%   on having observed the data y.
% 
% Notes: Newton-Raphson converges very quickly and is suggested in
% "Nonparametric Modeling of Hierarchically Exchangeable Data"
% by Peter Hoff
% 
% Authors: Jonathan Huang       (jch1 at cs.cmu.edu)
%          Tomasz Malisiewicz  (tomasz at cmu.edu)
flag = 1;
maxiterations = 15;
tol = 1e-9;
k = length(mu)+1;
z = initialz; %mu ;%+ 5*rand(length(mu),1);
n = sum(y);
edf = y/n;
edf = edf(1:end-1);
invcov = cov^(-1);
grad = zeros(k-1,1);
neghess = zeros(k-1,k-1);

change = tol+1;
its = 0;

% A few iterations of conjugate gradient to get things started
[z, fz, i] = minimize(z, @loglikefunc, 20,n,edf,mu,invcov);

% then some newton-raphson
tries = 0;
while its<maxiterations && change > tol
    oldz = z;
    
    pz = exp(z-max(z));
    pz = pz/(exp(-max(z))+sum(pz));
    
    grad = n*(edf - pz) - invcov*(z-mu);
    neghess = invcov + n*(diag(pz)) - n*(pz*(pz'));

    z = z+neghess^(-1)*grad;
    
    change = max(abs(z-oldz));
    its = its + 1;
    
    if its > maxiterations-1
        z = initialz + 2*(rand(length(mu),1)-ones(length(mu),1)*.5);
        %shh, buggy rand(length(mu),1);
        its = 0;
        tries = tries +1;
    end
    if tries > max(EMiter*5,20)
      flag = 0;
        disp('Giving up on Newton-Raphson - I am having a hard time finding a good initialization');
        z = initialz;
        its = maxiterations+30;
    end
end

muhat = z;
covhat = neghess^(-1);

loglike = n*dot(z,edf)-n*log(1+sum(exp(z)))-.5*(z-mu)'*invcov*(z-mu);

function [f,df]=loglikefunc(z,n,edf,mu,invcov)

pz = exp(z-max(z));
pz = pz/(exp(-max(z))+sum(pz));
f = -(n*dot(z,edf)-n*log(1+sum(exp(z)))-.5*(z-mu)'*invcov*(z-mu));
df = -(n*(edf - pz) - invcov*(z-mu));

