function [mu,cov] = hlnEM(Y)

% HLNEM
%
% function [mu,cov] = hlnEM(Y)
%
% input:
%   Y is a Nxk matrix where each row corresponds to a draw
%   from the hierarchical logistic normal.
% output:
%   mu, cov are fitted parameters (using Expectation-Maximization) 
%   to the Hierarchical Logistic Normal distribution.
%   mu is a (k-1)x1 vector, cov is a (k-1)x(k-1) covariance matrix.
% 
% This EM scheme is described by Peter Hoff in 
% "Nonparametric Modeling of Hierarchically Exchangeable Data"
% 
% Authors: Jonathan Huang       (jch1 at cs.cmu.edu)
%          Tomasz Malisiewicz  (tomasz at cmu.edu)

maxiterations = 25;
tol = 1e-6;

m = size(Y,1);
k = size(Y,2);
mu = zeros(k-1,1);
cov = eye(k-1,k-1);
muhat = zeros(k-1,m);
covhat = zeros(k-1,k-1,m);
perdocloglikes = zeros(m,1);
loglikes = [];
change = tol+1;
its = 0;
muhat = repmat(mu,1,m);

while its<maxiterations && change > tol
    oldmu = mu;
    oldcov = cov;
    for i=1:m
        [muhat(:,i),covhat(:,:,i),perdocloglikes(i),flags(i)] = ...
            conditionalMode(Y(i,:)',mu,cov,its,muhat(:,i));
    end
    
    mu = (1/m)*sum(muhat,2);
    cmuhat = muhat - repmat(mu,1,m);
    cov = (1/m)*(cmuhat*(cmuhat')+sum(covhat,3));
    
    change = max(max(abs(mu-oldmu)),max(max(abs(cov-oldcov))));
    its = its + 1;
    disp(sprintf('iteration: %d\t\tchange=%d',its,change));
end
